#ifndef MEASURETASK_H
#define MEASURETASK_H

#include "common.h"
#include "TaskQueue/BaseTask.h"
#include "laser/Laser.h"
#include "modbus/PLCMonitor.h"

#ifndef PI
#define PI 3.14159265
#endif

typedef struct PLANE
{
    float a;
    float b;
    float c;
}Plane;

enum ERROR_CODE
{
    eSucc=0
    ,eArea				//³µÎ»ÇøÓòŽíÎó
    ,eLidar				//À×ŽïµãÔÆ²»ÖØºÏ
    ,eCloud				//µãÔÆÎª¿Õ
    ,eNoCar				//ÎŽŒì²âµœ³µ
    ,eMulCar			//¶àÁŸ³µ
    ,eDistance			//³µŒäŸà¹ýÐ¡

    ,eLimitL=101
    ,eLimitT
    ,eLimitR
    ,eLimitB
};

typedef struct stCarPosition
{
    float x;
    float y;
    float a;
    float l;
    float w;
    float h;
    ERROR_CODE error_code;
    cv::RotatedRect rrect;
    int		pos_id;				//³µÎ»ID
    stCarPosition()
    {
        error_code = eSucc;
        pos_id = -1;
    }
}CarPosition;

std::string Error_string(ERROR_CODE code);
bool RegionInRegion(std::vector<cv::Point2f>& mini_poly, std::vector<cv::Point2f>& large_poly);
double distance_polys(std::vector<cv::Point2f> poly1, std::vector<cv::Point2f> poly2);

//typedef void(*ResultCallback)(Locate_Message message, void*);

class MeasureTask : public tq::BaseTask
{
public:
    MeasureTask(std::vector<CLaser*> lasers,void* locater,int position_id,
        modbus::CPLCMonitor* pcl, Automatic::stCalibParam	param);
    virtual void   Main();
    virtual void   Cancel();
    //void			SetResultCallback(void* ptr,ResultCallback func = NULL);

protected:
    static void		PushPoint(CPoint3D point, void* pointer);
    bool			IsLaserReady();
    ERROR_CODE		SelectResult(int plate_index, std::vector<CarPosition> results, CarPosition& result);
    //žùŸÝ³µÁŸÎ»ÖÃ·ÖÅä³µÎ»ID, ²»ÔÚ³µÎ»µÄ³µÁŸÐÅÏ¢ÉŸ³ý
    ERROR_CODE		Dispatch_posID(std::vector<CarPosition>& results);


protected:
    std::vector<CLaser*>		m_lasers;
    void*					m_locater;
    modbus::CPLCMonitor*		m_plc;

    int							m_position_id;		//³µÎ»id
    Automatic::stCalibParam		m_calib_param;		//ÓÐÐ§ÇøÓò
    PointCloudT					m_cloud;
    mutable std::mutex			m_mutex;


    static	std::mutex*			g_mutex;
    bool						m_exit;

    //CSearchPath					m_searchPath;
    //ResultCallback				m_post_result_func;
    void*						m_ptr;
};


#endif // MEASURETASK_H
